/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.container;

import java.util.Collections;
import java.util.Iterator;
import java.util.Map;

import woolpack.utils.KeyIteratorMap;

/**
 * 簡易のコンポーネントコンテナを表す{@link Map}。
 * スコープ制御のみを具備する。
 * @author nakamura
 *
 */
public class ScopeContainer extends KeyIteratorMap<String,Object>{
	/**
	 * スコープ管理されたオブジェクトを格納する{@link Map}の request と session 上の位置のデフォルト値。
	 */
	public static final String CONTAINER_KEY = "woolpack.container.SCOPE_CONTAINER";
	
	private final String attrName;
	private final Map<String,Object> request;
	private final Map<String,Object> session;
	private final Map<String,Object> application;
	private final Map<String,ComponentDef> componentDefs;
	
	/**
	 * コンストラクタ。
	 * @param attrName スコープ管理されたオブジェクトを格納する{@link Map}の request と session 上の位置。
	 * @param request Request スコープ。
	 * @param session Session スコープ。
	 * @param application Application スコープ。
	 * @param componentDefs 簡易コンポーネントコンテナの定義の{@link Map}。ステートレスであるべき。
	 * @throws StringIndexOutOfBoundsException attrNameが空の場合。
	 * @throws NullPointerException 引数のいずれかが null の場合。
	 */
	public ScopeContainer( 
			final String attrName,
			final Map<String,Object> request,
			final Map<String,Object> session,
			final Map<String,Object> application,
			final Map<String,ComponentDef> componentDefs){
		attrName.charAt(0);
		componentDefs.getClass();
		request.getClass();
		session.getClass();
		application.getClass();
		
		this.attrName = attrName;
		this.request = request;
		this.session = session;
		this.application = application;
		this.componentDefs = componentDefs;
	}
	
	/**
	 * コンストラクタ。
	 * スコープ管理されたオブジェクトを格納する{@link Map}の request と session 上の位置に{@link #CONTAINER_KEY}を使用する。
	 * @param request Request スコープ。
	 * @param session Session スコープ。
	 * @param application Application スコープ。
	 * @param componentDefs 簡易コンポーネントコンテナの定義の{@link Map}。ステートレスであるべき。
	 */
	public ScopeContainer(
			final Map<String,Object> request,
			final Map<String,Object> session,
			final Map<String,Object> application,
			final Map<String,ComponentDef> componentDefs){
		this(CONTAINER_KEY, request, session, application, componentDefs);
	}

	@Override
	protected Iterator<String> getKeyIterator() {
		return Collections.unmodifiableSet(componentDefs.keySet()).iterator();
	}

	@Override
	protected Object getValue(final Object key) {
		final ComponentDef componentDef = componentDefs.get(key);
		return componentDef.getComponent(attrName, (String)key, request, session, application);
	}
	
	/**
	 * @throws NullPointerException keyに対応する{@link ComponentDef}が定義されていない場合。
	 * @throws ClassCastException keyが文字列型でない場合。
	 * @throws UnsupportedOperationException いずれかのスコープへの設定に失敗した場合。
	 * @throws RuntimeException {@link ComponentDef#newInstance()}が投げた場合。
	 */
	@Override public Object get(final Object key){
		return getValue(key);
	}
}
